/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.ui.PageList");

dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.internal.PageRegistry");

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
var removeChildren = net.jazz.ajax.core.internal.util.removeChildren;
var PageRegistry = net.jazz.ajax.ui.internal.PageRegistry;
 
dojo.declare("net.jazz.ajax.ui.PageList", dijit._Widget, {
	
	// summary:
	//		unordered list of links for the pages in the current application.
	// description:
	//		This widget contains no public methods; simply instantiate it and
	//		place it within a Jazz Framework-based application. See
	//		resources/ui/PageList.css for information on styling the PageList.
	
	//
	// Internal and lifecycle methods
	//
	
	create: function(args, srcNodeRef) {
		if(srcNodeRef && srcNodeRef.tagName !== "UL") {
			throw new Error("PageList#create: 'srcNodeRef' must be a 'UL' element (was: " + srcNodeRef.tagName + ")");
		}
		if(!srcNodeRef) {
			srcNodeRef = document.createElement("ul");
		}
		PageList.superclass.create.call(this, args, srcNodeRef);
	},

	postCreate: function() {
		// subscribe to 'view page' events
		dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, "_selectPage");
		dojo.subscribe(net.jazz.ajax.internal.events.viewErrorPage, this, "_deselectCurrentPage");
		dojo.subscribe(net.jazz.ajax.internal.events.orderPages, this, "_reload");
		
		this._reload();
		
		// set the UL element's class to enable built-in styles and overridable
		// styles; see PageList.css for style details
		dojo.addClass(this.domNode, "net-jazz-ajax-PageList");
	},
	
	_reload: function() {
		removeChildren(this.domNode);
		
		// sort the pages, if necessary
		var pages = PageRegistry.getPages();
		if(this.sort) {
			console.warn("PageList no longer directly supports page ordering; see Workbench#orderPages");
		}
		
		// remove suppressed pages (if any) from the UI
		if(dojo.isArray(this.suppressedPages) && this.suppressedPages.length > 0){
			var map = {};
			dojo.forEach(this.suppressedPages, function(pageId){
				map[pageId] = true;
			});
			for(var i=pages.length-1; i>=0; i--){
				var pageId = pages[i].id;
				if(map[pageId]){
					pages.splice(i, 1);
				}
			}
		}
		// create the list and link elements
		var self = this;
		dojo.forEach(pages, function(page) {
			self._addPage(page);
		});
	},
	
	_addPage: function(page) {
		checkNotNull(page, "page");
		this.domNode.appendChild(this._createPageListElement(page));
	},
	
	_createPageListElement: function(page) {
		checkNotNull(page, "page");
		
		var liElement = document.createElement("li");
		
		liElement.appendChild(this._createPageLink(page));
		
		return liElement;
	},
	
	_createPageLink: function(page) {
		checkNotNull(page, "page");
		
		var linkElement = document.createElement("a");
		linkElement.href = page.getUri();
		linkElement.__jazzPageId = page.id;
		var innerWrapper = document.createElement("div");
		innerWrapper.className = "innerWrapper";
		var outerWrapper = document.createElement("div");
		outerWrapper.className = "outerWrapper";
		innerWrapper.appendChild(document.createTextNode(page.name));
		outerWrapper.appendChild(innerWrapper);
		linkElement.appendChild(outerWrapper);
		return linkElement;
	},
	
	_selectPage: function(page) {
		// summary:
		//		Marks the page link for 'id' as selected, and deselects other
		//      page links.
		// id:
		//		The id of the page to select.
		if (!page || !dojo.isString(page.id)) {
			throw new Error("PageList#_handleViewPage: Illegal Argument");
		}
		dojo.forEach(this.domNode.getElementsByTagName("A"), function(link) {
			link.className = link.__jazzPageId === page.id ? "selected" : "";
		});
	},
	_deselectCurrentPage: function() {
		dojo.forEach(this.domNode.getElementsByTagName("A"), function(link) {
			link.className = "";
		});
	}
});

var PageList = net.jazz.ajax.ui.PageList;

})();