/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.UserEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.io.iframe");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.PasswordChangeDialog");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var LicenseAdminClient = com.ibm.team.repository.web.client.internal.licensing;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MANAGER = "com.ibm.team.repository.manageUsers";
var USER_EDITOR = "com.ibm.team.repository.editUser";
var PHOTO_SERVICE_URI = "com.ibm.team.repository.service.internal.IMemberPhotoService";
var MSG_COOKIE_NAME = "com.ibm.team.repository.editUser#saveMsg";
var FILE_UPLOAD_RESPONSE = "File Upload Response:";
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var MANAGE_ARCHIVED_LINK = "#action=com.ibm.team.repository.manageArchivedUsers";
var MANAGE_ACTIVE_LINK = "#action=com.ibm.team.repository.manageUsers";
var REG_GROUP_ERROR_FLAG = "REG_GROUP_ERROR:";
var USER_NOT_FOUND_MSG_ID = "CRJAZ1231I";

var _currentBindRequest;
var _currentUploadFrameNumber = 100;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.UserEditor", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/UserEditor.html"),
	
	constructor: function(args){
		this.SERVICE_PREFIX = net.jazz.ajax._contextRoot + "/service/";
		this.isNew = args.itemId === "new";
		if (!this.isNew) {
			this.itemId = args.itemId;
		}
		this._defaultPhotoSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/no-photo.png");
		this._removeIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
		this._removeIconHoverSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove-hov.gif");
		this._revertIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore.gif");
		this._revertIconHoverSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore-hov.gif");
		this._userIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/userIcon.png");
		this._noCALMessage = "A user that has not been assigned a Client Access License has read-only access to all capabilities. A Client Access License is also not required for a user to administer a Jazz Team Server. The server does not limit the number of users with no Client Access License assigned - they will not be counted as part of the server limit.";
		this._serverLicenseLimitMessage = LicenseAdminClient.SERVER_COUNT_MAX_MSG;
	},
	
	postCreate: function () {
		this.connectHandles = [];
		this.inputFields = [
			this.name,
			this.userId,
			this.emailAddress
		];
		this.connect(this.name, "onkeypress", function (e) {
			if (e.keyCode === KEY_RETURN || e.keyCode === KEY_ENTER) {
				dojo.stopEvent(e);
				this.togglePrimaryField("Preview");
			}
		});
		this.connect(this.photoFileLocation,"onchange","handlePhotoChange");
		this.connect(this.photoRemoveButton,"onclick", function (e) {
			if (!e.target.clickDisabled) {
				this.removePhoto();
			}
		});
		this.connect(this.photoRevertButton,"onclick", function (e) {
			if (!e.target.clickDisabled) {
				this.revertPhoto();
			}
		});
		this.connect(this.saveButton,"onclick","handleSave");
		this.connect(this.photoRevertButton, "onmouseover", function (e) {
			if (!e.target.clickDisabled) {
				e.target.src = this._revertIconHoverSrc;
			}
		});
		this.connect(this.photoRevertButton, "onmouseout", function (e) {
			e.target.src = this._revertIconSrc;
		});
		this.connect(this.photoRemoveButton, "onmouseover", function (e) {
			if (!e.target.clickDisabled) {
				e.target.src = this._removeIconHoverSrc;
			}
		});
		this.connect(this.photoRemoveButton, "onmouseout", function (e) {
			e.target.src = this._removeIconSrc;
		});
		this.connect(this.previewName,"onmouseover", function (e) {
			e.target.style.backgroundColor = "#FFFFCC";
		});
		this.connect(this.previewName,"onmouseout", function (e) {
			e.target.style.backgroundColor = "";
		});
		this.connect(this.previewName,"onclick", function (e) {
			this.togglePrimaryField("Edit");
		});
		this.connect(this.name, "onblur", function (e) {
			this.togglePrimaryField("Preview");
		});
		
		for (var i=0; i < this.inputFields.length; i++) {
			this.connect(this.inputFields[i],"onkeyup","handleChange");
			this.connect(this.inputFields[i],"onchange","handleChange");
		}
		this.connect(this.changePassword,"onclick","handlePasswordChange");
		this.refresh();
	},
	
	refresh: function () {
		if (!this.navigateAwayHandle) {
			this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);
		}
		this.clearState();
		if (this.isNew) {
			this.name.value = "User Name";
			this.togglePrimaryField("Edit");
			setTimeout(dojo.hitch(this.name,"select"), 250);
		}
		this.getUserRegistryInfo();
		this.checkForSuccessMsg();
	},
	
	_handleNavigate: function(urlObj) {
		if (this.changesPending()) {
			if (confirm("Are you sure you want to navigate away from this page?\n\nYou have unsaved changes in the user editor.  If you continue, you will lose your changes.\n\nPress OK to continue, or Cancel to stay on the current page.")) {
				PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				this.navigateAwayHandle = null;
				PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
				this.onBeforeUnloadHandle = null;
				return false;
			}
			else {
				return true;
			}
		}
		return false;
	},
	
	navigateAwayEvent: function() {
		return this.changesPending() ? "You have unsaved changes in the user editor.  If you continue, you will lose your changes." : false;
	},
	
	checkForSuccessMsg: function () {
		var msgCookie = dojo.cookie(MSG_COOKIE_NAME);
		if (msgCookie) {
			this.displaySuccessMessage(msgCookie, true);
			var now = new Date();
			now.setTime(now.getTime() - 1);
			dojo.cookie(MSG_COOKIE_NAME,"",{expires: now});
		}
	},
	
	clearState: function () {
		this.name.value = "";
		this.userId.value = "";
		this.emailAddress.value = "";
		this.changePassword.disabled = true;
		this.changePassword.style.display = "none";
		dojox.data.dom.removeChildren(this.repoPermissions);
		dojox.data.dom.removeChildren(this.photoArea);
		this.removePhotoFlag = false;
		this.photoFileLocation.value = "";
		dojo.toggleClass(this.name, "highlightChange", false);
		dojo.toggleClass(this.userId, "highlightChange", false);
		dojo.toggleClass(this.emailAddress, "highlightChange", false);
		dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
		for (var i=0; i < this.inputFields.length; i++) {
			this.inputFields[i].changePending = false;
		}
		this.photoRevertButton.clickDisabled = true;
		this.setDirty(false);
		dojox.data.dom.removeChildren(this.messageBox);
		dojox.data.dom.removeChildren(this.longMessageBox);
		this.messageBox.className = "";
		this.longMessageBox.className = "";
	},
	
	getUserRegistryInfo: function () {
		var srh = new ServiceResponseHandler(this, "getUserRegistryInfoCallback", "handleRegistryInfoError");
		AdminClient.getUserRegistryInfo(srh);
	},
	
	handleRegistryInfoError: function (err) {
		/*
		 * The error message CRJAZ0799I indicates that the user registry is not supported.  We can't query or write to the external
		 * registry, but we should allow repository-only creation and editing of users.
		 */
		if (err.message.indexOf("CRJAZ0799I") !== -1) {
			var userRegistryInfo = {
				writable: false,
				queryable: false
			};
			this.getUserRegistryInfoCallback(userRegistryInfo);
		}
		else {
			var msg = document.createElement("div");
			msg.appendChild(document.createTextNode("Error fetching user registry information.  Please check your user registry configuration."));
			if (err.message != null && err.message != "") {
				msg.appendChild(document.createElement("br"));
				msg.appendChild(document.createElement("br"));
				msg.appendChild(document.createTextNode(err.message));
			}
			this.displayLongErrorMessage(msg);
		}
	},
	
	getUserRegistryInfoCallback: function (userRegistryInfo) {
		this.userRegistryInfo = new Object();
		this.userRegistryInfo.isWritable = userRegistryInfo.writable;
		this.userRegistryInfo.isQueryable = userRegistryInfo.queryable;
		this.userRegistryInfo.fields = new Object();
		if (userRegistryInfo.readOnlyAttributes != null && userRegistryInfo.readOnlyAttributes.length > 0) {
			for (var i = 0; i < userRegistryInfo.readOnlyAttributes.length; i++) {
				this.userRegistryInfo.fields[userRegistryInfo.readOnlyAttributes[i]] = { writable: false };
			}
		}
		for (var i = 0; i < this.inputFields.length; i++) {
			if (this.userRegistryInfo.fields[this.inputFields[i].name] == null) {
				this.userRegistryInfo.fields[this.inputFields[i].name] = { writable: true };
			}
		}
		
		if (!this.isNew) {
			for (var i=0; i < this.inputFields.length; i++) {
				if (!this.userRegistryInfo.fields[this.inputFields[i].name].writable) {
					this.inputFields[i].disabled = true;
				}
			}
			if (userRegistryInfo.writable) {
				this.changePassword.style.display = "";
			}
		}
		else {
			if (this.userRegistryInfo.isWritable) {
				this.displayLongWarningMessage("New users will be created with a default password equal to their User Id in your directory service.");
			}
		}

		dojox.data.dom.removeChildren(this.repoPermissions);
		this.repoPermissions.appendChild(document.createTextNode("Loading..."));
		var srh = new ServiceResponseHandler(this, "populateRoles", "handleRegistryGroupError");
		AdminClient.getAllRoles(srh);
	},
	
	getLicensingInfo: function () {
		var srh = new ServiceResponseHandler(this, "getLicensingInfoCallback", "handleLicensingInfoError");
		LicenseAdminClient.getLicenseInfo(srh);
	},
	
	getLicensingInfoCallback: function (data) {
		this.licenseInputs = [];
		dojox.data.dom.removeChildren(this.licensesDiv);
		if (data.serverLicense != null) {
			this.isTrialLicense = data.serverLicense.expirationTime ? true : false;
		}
		if (data.types != null) {
			var licenses = LicenseAdminClient.sortCALs(data.types);
			for (var i=0; i < licenses.length; i++) {
				var licenseInfo = LicenseAdminClient.getClientAccessLicenseInfo(data, licenses[i]);
				var div = document.createElement("div");
				var checkbox = document.createElement("input");
				checkbox.type = "checkbox";
				checkbox.tabIndex = 3;
				checkbox.id = "checkBox_" + licenses[i].id;
				checkbox.value = licenses[i].id;
				var availableLicenses = licenseInfo.available;
				if (availableLicenses === 0 || availableLicenses === -2) {
					checkbox.disabled = "disabled";
				}
				this.licenseInputs.push(checkbox);
				var curLabel = document.createElement("label");
				curLabel.htmlFor = checkbox.id;
				curLabel.appendChild(document.createTextNode(licenseInfo.name + " "));
				var licenseCountSpan = document.createElement("span");
				licenseCountSpan.appendChild(document.createTextNode(licenseInfo.availabilityLabel));
				licenseCountSpan.licenseCount = availableLicenses;
				licenseCountSpan.assignableToUserWithCALs = licenseInfo.assignableToUserWithCALs;
				licenseCountSpan.assignableToUserWithoutCALs = licenseInfo.assignableToUserWithoutCALs;
				checkbox.licenseCountSpan = licenseCountSpan;
				curLabel.appendChild(licenseCountSpan);
				div.appendChild(checkbox);
				div.appendChild(curLabel);
				dojo.toggleClass(div, "checkboxDiv", true);
				this.licensesDiv.appendChild(div);
				var span = document.createElement("span");
				this.licensesDiv.appendChild(span);
				var tooltip = new HoverPopup({
					connectNode: [ curLabel.parentNode ],
					label: licenses[i].description,
					showDelay: 300,
					orientation: "L"
				}, span);
				this.connect(checkbox,"onclick","handleChange");
				this.connect(checkbox,"onclick", function(e) {
					if (e.target.licenseCountSpan.licenseCount != -1) {
						if (e.target.checked) {
							// Only update the available license count if the license is not "unlimited" or "disabled"
							if (e.target.licenseCountSpan.licenseCount !== -1 && e.target.licenseCountSpan.licenseCount !== -2) {
								dojox.data.dom.removeChildren(e.target.licenseCountSpan);
								e.target.licenseCountSpan.appendChild(document.createTextNode("(" + --e.target.licenseCountSpan.licenseCount + " available)"));
							}
							this.noLicenseWarning.style.display = "none";
						}
						else {
							// Only update the available license count if the license is not "unlimited" or "disabled"
							if (e.target.licenseCountSpan.licenseCount !== -1 && e.target.licenseCountSpan.licenseCount !== -2) {
								dojox.data.dom.removeChildren(e.target.licenseCountSpan);
								e.target.licenseCountSpan.appendChild(document.createTextNode("(" + ++e.target.licenseCountSpan.licenseCount + " available)"));
							}
							var assigned = false;
							for (var i=0; i < this.licenseInputs.length; i++) {
								if (this.licenseInputs[i].checked) {
									assigned = true;
									break;
								}
							}
							this.noLicenseWarning.style.display = assigned ? "none" : "";
						}
					}
				});
			}
			if (!this.isNew) {
				var args = {"userId": this.userId.savedValue};
				var srh = new ServiceResponseHandler(this, "getAssignedLicensesCallback", "handleAssignedLicensesError");
				LicenseAdminClient.getAssignedLicenses(srh, args);
			}
			else {
				this.noLicenseWarning.style.display = "";
			}
		}
		else {
			var div = document.createElement("div");
			div.style.color = "#999999";
			div.appendChild(document.createTextNode("No client access licenses were found."));
			this.licensesDiv.appendChild(div);
		}
	},
	
	handleLicensingInfoError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching license information from the repository."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	handleRegistryGroupError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching group information from the user registry.  Please check your registry configuration."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	populateRoles: function (data) {
		dojox.data.dom.removeChildren(this.repoPermissions);
		this.roleInputs = new Array();
		if (!this.userRegistryInfo.isWritable) {
			var div = document.createElement("div");
			div.appendChild(document.createTextNode("Notice: You are using a directory service that is not writable.  User roles cannot be modified."));
			div.style.marginBottom = "1em";
			this.repoPermissions.appendChild(div);
		}
		for (var i=0; i < data.length; i++) {
			var div = document.createElement("div");
			var checkbox = document.createElement("input");
			var checkBoxId = data[i].name + "CheckBox"
			checkbox.id = checkBoxId;
			checkbox.type = "checkbox";
			checkbox.tabIndex = 3;
			checkbox.name = "repoPermissions";
			checkbox.value = data[i].name;
			checkbox.savedValue = false;
			checkbox.style.marginRight = "0.5em";
			if (!this.userRegistryInfo.isWritable) {
				checkbox.disabled = true;
			}
			this.roleInputs.push(checkbox);
			this.connect(checkbox,"onclick","handleChange");
			div.appendChild(checkbox);
			var boxLabel = document.createElement("label");
			boxLabel.appendChild(document.createTextNode(data[i].name));
			boxLabel.htmlFor = checkBoxId;
			checkbox.labelElem = boxLabel;
			div.appendChild(boxLabel);
			var span = document.createElement("span");
			div.appendChild(span);
			var tooltip = new HoverPopup({
				connectNode: [ boxLabel.parentNode ],
				label: data[i].description,
				showDelay: 300,
				orientation: "L"
			}, span);
			div.style.marginBottom = "0.3em";
			this.repoPermissions.appendChild(div);
		}
		if (!this.isNew) {
			var srh = new ServiceResponseHandler(this, "populateUserData", "handleUserDataError");
			AdminClient.getContributorByUUID({uuid: this.itemId}, srh);
		}
		else {
			var jazzUsersBox = dojo.byId("JazzUsersCheckBox");
			if (jazzUsersBox && this.userRegistryInfo && this.userRegistryInfo.isWritable) {
				jazzUsersBox.checked = true;
			}
			this.name.savedValue = "";
			this.userId.savedValue = "";
			this.emailAddress.savedValue = "";
			this.photoExists = false;
			this.photoRemoveButton.clickDisabled = !this.photoExists;
			this.showSavedPhoto();
			this.getLicensingInfo();
		}
	},
	
	handleUserDataError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching user data.  Please check your registry configuration."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	togglePrimaryField: function (mode) {
		if (mode === "Preview") {
			this.name.style.display = "none";
			dojox.data.dom.removeChildren(this.previewName);
			this.previewName.appendChild(document.createTextNode(this.name.value || "User Name"));
			this.previewName.style.display = "";
		}
		else if (mode === "Edit") {
			this.previewName.style.display = "none";
			this.name.style.display = "";
			this.name.focus();
			this.name.select();
			dojox.data.dom.removeChildren(this.messageBox);
			this.messageBox.className = "message shortMsg";
		}
	},
	
	populateUserData: function (data) {
		this.showBreadcrumbs(data.archived);
		this.name.value = data.name;
		this.name.savedValue = data.name;
		this.togglePrimaryField("Preview");
		this.userId.value = data.userId;
		this.userId.savedValue = data.userId;
		this.emailAddress.value = data.emailAddress;
		this.emailAddress.savedValue = data.emailAddress;
		this.photoExists = data.photoExists;
		this.photoRemoveButton.clickDisabled = !this.photoExists;
		this.showSavedPhoto();
		if (com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId == data.userId) {
			this.changePassword.disabled = false;
		}
		if (data.roles != null) {
			if (data.roles.length === 1 && data.roles[0].indexOf(REG_GROUP_ERROR_FLAG) === 0) {
				var message = data.roles[0].substring(REG_GROUP_ERROR_FLAG.length);
				if (message.indexOf(USER_NOT_FOUND_MSG_ID) === 0) {
					this.displayLongWarningMessage("This user was not found in the directory service. This user will not be able to login unless they have an account in the directory service.");
				}
				else {
					this.displayLongWarningMessage("Problem fetching user roles: " + message);
				}
			}
			else {
				for (var i=0; i < data.roles.length; i++) {
					for (var j=0; j < this.roleInputs.length; j++) {
						if (this.roleInputs[j].value == data.roles[i]) {
							this.roleInputs[j].checked = true;
							this.roleInputs[j].savedValue = true;
						}					
					}
				}
			}
		}
		this.getLicensingInfo();
	},
	
	showBreadcrumbs: function (isArchived) {
		dojox.data.dom.removeChildren(this.breadcrumbs);
		var link = document.createElement("a");
		link.href = isArchived ? MANAGE_ARCHIVED_LINK : MANAGE_ACTIVE_LINK;
		link.appendChild(document.createTextNode(isArchived ? "Archived Users" : "Active Users"));
		this.breadcrumbs.appendChild(link);
		var span = document.createElement("span");
		span.appendChild(document.createTextNode(">"));
		this.breadcrumbs.appendChild(span);
	},
	
	getAssignedLicensesCallback: function (data) {
		if (data == null) {
			data = new Array();
		}
		for (var i=0; i < this.licenseInputs.length; i++) {
			this.licenseInputs[i].savedValue = false;
			for (var j=0; j < data.length; j++) {
				if (data[j].id == this.licenseInputs[i].value) {
					this.licenseInputs[i].savedValue = true;
					this.licenseInputs[i].checked = true;
					this.licenseInputs[i].disabled = false;
				}
			}
		}
		if (data.length === 0) {
			this.noLicenseWarning.style.display = "";
			for (var i=0; i < this.licenseInputs.length; i++) {
				var disable = !this.licenseInputs[i].licenseCountSpan.assignableToUserWithoutCALs;
				this.licenseInputs[i].disabled = disable;
				if (disable && this.licenseInputs[i].licenseCountSpan.assignableToUserWithCALs) {
					this.serverLicenseLimitMessage.style.display = "";
				}
			}
		}
	},
	
	handleAssignedLicensesError: function (err) {
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Error fetching assigned licenses from the repository."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	handlePasswordChange: function (e) {
		dojo.stopEvent(e);
		this.passwordChangeDialog = new com.ibm.team.repository.web.ui.internal.admin.users.PasswordChangeDialog(
				{userName: this.name.savedValue, userId: this.userId.savedValue, itemId: this.itemId});
		this.connect(this.passwordChangeDialog.cancelButton, "onclick", function (e) {
			dojo.stopEvent(e);
			this.passwordChangeDialog.close();
		});
		this.connect(this.passwordChangeDialog.finishButton, "onclick", function (e) {
			dojo.stopEvent(e);
			if (this.passwordChangeDialog.newPassword.value != this.passwordChangeDialog.confirmPassword.value) {
				alert("The passwords you typed do not match");
				this.passwordChangeDialog.newPassword.focus();
			}
			else {
				var srh = new ServiceResponseHandler(this,"passwordChangeCallback","passwordChangeError");
				AdminClient.changeUserPassword({
					userId: this.passwordChangeDialog.userId,
					oldPassword: this.passwordChangeDialog.oldPassword.value,
					newPassword: this.passwordChangeDialog.newPassword.value
				}, srh);
			}
		});
	},
	
	passwordChangeCallback: function () {
		this.displaySuccessMessage("Password changed successfully.", true);
		this.passwordChangeDialog.close();
	},
	
	passwordChangeError: function (err) {
		if (err.status == 401) {
			this.passwordChangeDialog.showErrorMessage("Error changing password: Incorrect password");
			this.passwordChangeDialog.oldPassword.focus();
		}
		else {
			this.passwordChangeDialog.showErrorMessage("Error changing password: " + err.message);
		}
	},
	
	handleChange: function (e) {
		if (e.target.type == "checkbox") {
			if (e.target.checked != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirty(true);
			}
			else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirty(false);
				}
			}
		}
		else {
			if (e.target.value != e.target.savedValue) {
				e.target.changePending = true;
				this.setDirty(true);
			}
			else {
				e.target.changePending = false;
				if (!this.changesPending()) {
					this.setDirty(false);
				}
			}
		}
	},
	
	setDirty: function (isDirty) {
		this.saveButton.disabled = !isDirty;
		if (isDirty) {
			this.modifiedFlag.style.display = "inline";
		}
		else {
			this.modifiedFlag.style.display = "none";
		}
	},
	
	setPhoto: function (uri, msg, highlightMsg) {
		if (uri) {
			this.photoArea.src = uri;
		}
		else {
			this.photoArea.src = this._defaultPhotoSrc;
		}
	},
	
	handlePhotoChange: function () {
		if (this.photoFileLocation.value != "") {
			this.removePhotoFlag = false;
			this._performUpload(this.photoFileLocation.value);
			this.photoRevertButton.clickDisabled = false;
			this.photoRemoveButton.clickDisabled = false;
			dojo.toggleClass(this.photoFileLocation, "highlightChange", true);
			this.setDirty(true);
		}
		else {
			dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
			if (!this.changesPending()) {
				this.setDirty(false);
			}
			if (this.removePhotoFlag) {
				this.removePhoto();
				this.photoRemoveButton.clickDisabled = true;
				this.photoRevertButton.clickDisabled = false;
			}
			else {
				this.showSavedPhoto();
				this.photoRemoveButton.clickDisabled = !this.photoExists;
				this.photoRevertButton.clickDisabled = true;
			}
		}
	},
	
	_performUpload: function(fullPath /*e, isRestart*/) {
		this._isUploadInProgress = true;
		this._priorFullPath = fullPath;
		var parts= fullPath.split("\\");
		var fileName= parts[parts.length-1];

		var self= this;
		if(this._currentBindRequest != null) {
			this._currentBindRequest.cancel();
			this._currentBindRequest  = null;
		}
		
/*
 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
 * next one breaks. Resetting the iframe fixes the problem
 * Unfortunately this code reaches into iframe private variables.
 */
		if(dojo.io.iframe._frame) {
			document.body.removeChild(dojo.io.iframe._frame);				
		}
		frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
		dojo.io.iframe._iframeName = frameName;
		dojo.io.iframe._frame = null;
		
		this._currentBindRequest = dojo.io.iframe.send({
			form: this._uploadForm,
			url: TeamServerClient._getServicePrefix() + PHOTO_SERVICE_URI, //+ "?projectId=" + getCurrentProjectArea().itemId,
			content: {
					mimetype: "text/html",
					fileFields: "photoFileLocation"
				},
			handleAs:  "html",
			handle: function(response, ioArgs) {
				self.isUploadInProgress = "false";
				if(response instanceof Error){
					if(response.dojoType && response.dojoType == "cancel") {
						return; // this is a cancelled request, ignore it
					}
					self._currentBindRequest = null; // no need to cancel it any more
					alert("An error occured during file upload.\nThis will happen when an invalid fully qualified file name is provided.");
					console.debug("Error uploading file: " + response.message);
				}
				else {
					var responseMsg = response.body.innerHTML;
					if (responseMsg.indexOf(FILE_UPLOAD_RESPONSE) == 0) {
						var data = responseMsg.substring(FILE_UPLOAD_RESPONSE.length);
						var dataArray = data.split(",");
						var photoLocation = dataArray[0];
						var photoSize = dataArray[1];
						var photoContentType = dataArray[2];
						var photoChecksum = dataArray[3];
						self.photoFileLocation.savedLocation = photoLocation;
						self.photoFileLocation.savedFileSize = photoSize;
						self.photoFileLocation.savedContentType = photoContentType;
						self.photoFileLocation.savedChecksum = photoChecksum;
						self.removePhotoFlag = false;
						self.setPhoto(self.SERVICE_PREFIX + PHOTO_SERVICE_URI + "?photoLocation=" + photoLocation);
					}
				}
			}
		});
	},
	
	showSavedPhoto: function () {
		if (this.photoExists) {
			this.setPhoto(this.SERVICE_PREFIX + PHOTO_SERVICE_URI + "?contributorItemId=" + this.itemId);
		}
		else {
			this.setPhoto(null,"No photo on file");
		}
	},
	
	removePhoto: function (e) {
		if (e != null) {
			dojo.stopEvent(e);
		}
		this.photoFileLocation.value = "";
		this.photoFileLocation.savedLocation = null;
		this.photoFileLocation.savedFileSize = null;
		this.photoFileLocation.savedContentType = null;
		this.photoFileLocation.savedChecksum = null;
		dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
		if (this.photoExists) {
			this.removePhotoFlag = true;
			this.setPhoto(null,"Photo to be removed on Save",true);
			this.photoRemoveButton.clickDisabled = true;
			this.photoRevertButton.clickDisabled = false;
			this.setDirty(true);
		}
		else {
			this.showSavedPhoto();
			this.photoRemoveButton.clickDisabled = true;
			this.photoRevertButton.clickDisabled = true;
			this.setDirty(this.changesPending());
		}
	},
	
	revertPhoto: function (e) {
		if (e != null) {
			dojo.stopEvent(e);
		}
		this.removePhotoFlag = false;
		this.photoFileLocation.value = "";
		dojo.toggleClass(this.photoFileLocation, "highlightChange", false);
		this.showSavedPhoto();
		this.photoRemoveButton.clickDisabled = !this.photoExists;
		this.photoRevertButton.clickDisabled = true;
		this.setDirty(this.changesPending());
	},
	
	handleSave: function (e) {
		dojo.stopEvent(e);
		// If a user is editing their own account and they have revoked privilges from themselves, display a warning before continuing
		if (com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId == this.userId.savedValue) {
			for (var i=0; i < this.roleInputs.length; i++) {
				var adminChecked, usersChecked, adminRevoked, usersRevoked;
				if (this.roleInputs[i].value == "JazzAdmins") {
					if (this.roleInputs[i].checked) {
						adminChecked = true;
						adminRevoked = false;
					}
					else {
						adminChecked = false;
						if (this.roleInputs[i].savedValue) {
							adminRevoked = true;
						}
						else {
							adminRevoked = false;
						}
					}
				}
				if (this.roleInputs[i].value == "JazzUsers") {
					if (this.roleInputs[i].checked) {
						usersChecked = true;
						usersRevoked = false;
					}
					else {
						usersChecked = false;
						if (this.roleInputs[i].savedValue) {
							usersRevoked = true;
						}
						else {
							usersRevoked = false;
						}
					}
				}
			}
			var msg = "";
			if (adminRevoked) {
				if (!usersChecked) {
					msg = "Warning: You are about to revoke JazzAdmin and JazzUser priviliges from your account.  If you continue, your account will be completely locked out of the repository.  Are you sure you would like to continue?";
				}
				else {
					msg = "Warning: You are about to revoke JazzAdmin privileges from your account.  If you continue, your account will be locked out of the admin application.  Are you sure you would like to continue?";
				}
			}
			else if (!adminChecked && usersRevoked) {
				msg = "Warning: You are about to revoke JazzUser privileges from your account.  If you continue, your account will be completely locked out of the repository.  Are you sure you would like to continue?";
			}
			if (msg != "" && !confirm(msg)) {
				return;
			}
		}
		this.saveUser();
	},
	
	saveUser: function () {
		this.saveButton.disabled = true;
		var saveObj = {
			itemId: this.itemId
		};
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.isNew || this.inputFields[i].changePending) {
				saveObj[this.inputFields[i].name] = this.inputFields[i].value;
			}
		}
		var repoRoles = new Array();
		for (var i=0; i < this.roleInputs.length; i++) {
			if (this.roleInputs[i].checked) {
				repoRoles.push(this.roleInputs[i].value);
			}
		}
		saveObj["jsonRoles"] = dojo.toJson(repoRoles);
		var licenseChanges = {
			add: [],
			remove: []
		};
		if (this.licenseInputs) {
			for (var i=0; i < this.licenseInputs.length; i++) {
				if (this.licenseInputs[i].changePending) {
					licenseChanges[(this.licenseInputs[i].checked ? "add" : "remove")].push(this.licenseInputs[i].value);
				}
			}
		}
		saveObj["jsonLicenses"] = dojo.toJson(licenseChanges);
		if (this.photoFileLocation.savedLocation != null) {
			saveObj["photoLocation"] = this.photoFileLocation.savedLocation;
			saveObj["photoSize"] = this.photoFileLocation.savedFileSize;
			saveObj["photoContentType"] = this.photoFileLocation.savedContentType;
			saveObj["photoChecksum"] = this.photoFileLocation.savedChecksum;
		}
		else if (this.removePhotoFlag) {
			saveObj["photoLocation"] = "";
		}
		if (!this.userRegistryInfo.isWritable) {
			saveObj["repoOnly"] = true;
		}
		var msg = this.validateRequired(saveObj);
		if (msg != null && msg != "") {
			this.displayErrorMessage(msg);
		}
		else {
			var srh = new ServiceResponseHandler(this, "handleSaveResponse", "handleSaveError");
			AdminClient.postContributor(saveObj, srh);
			this.displayPendingMessage("Save in progress...");
		}
	},
	
	validateRequired: function (saveData) {
		if (saveData.name == "" || (this.isNew && saveData.name == null)) {
			this.togglePrimaryField("Edit");
			return "The user must have a name";
		}
		if (saveData.userId == "" || (this.isNew && saveData.userId == null)) {
			this.userId.focus();
			return "The user must have a User Id";
		}
		if (saveData.emailAddress == "" || (this.isNew && saveData.emailAddress == null)) {
			this.emailAddress.focus();
			return "The user must have an e-mail address";
		}
	},
	
	handleSaveResponse: function(response) {
		if (this.isNew && response) {
			dojo.cookie(MSG_COOKIE_NAME, "User created successfully.");
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
			this.navigateAwayHandle = null;
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
			this.onBeforeUnloadHandle = null;
			document.location.href = "#action=" + USER_EDITOR + "&itemId=" + response;
		}
		else {
			this.refresh();
			this.displaySuccessMessage("User saved successfully.", true);
		}
	},
	
	handleSaveError: function (err) {
		this.saveButton.disabled = false;
		var msg = document.createElement("div");
		msg.appendChild(document.createTextNode("Saving user failed.  Please see your server log for more detail."));
		if (err.message != null && err.message != "") {
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createElement("br"));
			msg.appendChild(document.createTextNode(err.message));
		}
		this.displayLongErrorMessage(msg);
	},
	
	onRefresh: function(e){
		dojo.stopEvent(e);
		if (this.changesPending() && !confirm("Refreshing will discard any changes you have made.  Would you like to continue?")) {
			return;
		}
		this.refresh();
	},
	
	changesPending: function () {
		if (this.removePhotoFlag) {
			return true;
		}
		if (this.photoFileLocation.value != "") {
			return true;
		}
		for (var i=0; i < this.inputFields.length; i++) {
			if (this.inputFields[i].changePending) {
				return true;
			}
		}
		if (this.roleInputs) {
			for (var i=0; i < this.roleInputs.length; i++) {
				if (this.roleInputs[i].changePending) {
					return true;
				}
			}
		}
		if (this.licenseInputs) {
			for (var i=0; i < this.licenseInputs.length; i++) {
				if (this.licenseInputs[i].changePending) {
					return true;
				}
			}
		}
		return false;
	},
	
	displaySuccessMessage: function (msg, showManagerLink) {
		this.displayMessage(msg,"successMsg");
		if (showManagerLink) {
			this.messageBox.appendChild(document.createTextNode(" "));
			var link = document.createElement("a");
			link.href = "#action=" + MANAGER;
			link.appendChild(document.createTextNode("Return to Manage Users"));
			this.messageBox.appendChild(link);
		}
	},
	
	displayPendingMessage: function (msg) {
		this.displayMessage(msg, "pendingMsg");
	},
	
	displayWarningMessage: function (msg) {
		this.displayMessage(msg, "warningMsg");
	},
	
	displayErrorMessage: function (msg) {
		this.displayMessage(msg, "errorMsg");
	},
	
	displayMessage: function (msg, cssClass) {
		dojox.data.dom.removeChildren(this.messageBox);
		dojox.data.dom.removeChildren(this.longMessageBox);
		this.longMessageBox.className = "";
		this.messageBox.className = "message shortMsg " + cssClass;
		this.messageBox.appendChild(document.createTextNode(msg));
	},
	
	displayLongSuccessMessage: function (msg) {
		if (dojo.isString(msg)) {
			msg = document.createTextNode(msg);
		}
		this.displayLongMessage(msg,"successMsg");
	},
	
	displayLongPendingMessage: function (msg) {
		if (dojo.isString(msg)) {
			msg = document.createTextNode(msg);
		}
		this.displayLongMessage(msg,"pendingMsg");
	},
	
	displayLongWarningMessage: function (msg) {
		if (dojo.isString(msg)) {
			msg = document.createTextNode(msg);
		}
		this.displayLongMessage(msg,"warningMsg");
	},
	
	displayLongErrorMessage: function (msg) {
		if (dojo.isString(msg)) {
			msg = document.createTextNode(msg);
		}
		this.displayLongMessage(msg, "errorMsg");
	},
	
	displayLongMessage: function (msg, cssClass) {
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = "";
		dojox.data.dom.removeChildren(this.longMessageBox);
		this.longMessageBox.className = "message longMsg " + cssClass;
		this.longMessageBox.appendChild(msg);
	},
	
	destroy: function () {
		dojo.forEach(this.connectHandles, function (handle) {
			dojo.disconnect(handle);
		});
		this.inherited(arguments);
	}
});

})();